using System;
using System.Collections.Generic;
using System.Text;

using Entities;
using System.Collections;

namespace Domain.Implementation
{
    public class Order : BaseEntity, IOrder
    {
        public virtual event EventHandler<OrderAmountChangedEventArgs> AmountChanged;
        protected void OnAmountChanged(float delta)
        {
            if (this.AmountChanged != null)
                this.AmountChanged(this, new OrderAmountChangedEventArgs(delta));
        }

        private float amount;
        public virtual float Amount
        {
            get { return amount; }
            set { amount = value; }
        }

        private IEntity customer;
        public virtual IEntity Customer
        {
            get { return customer; }
            set { customer = value; }
        }

        protected IList orderLines = new List<OrderLine>();
        public virtual IEnumerable<IOrderLine> OrderLines
        {
            get
            {
                List<IOrderLine> result = new List<IOrderLine>(this.orderLines.Count);
                foreach (IOrderLine ol in this.orderLines)
                    result.Add(ol);

                return result;
            }
        }


        public virtual void AddProduct(IProduct p, float quantity)
        {
            if (quantity == 0)
                return;

            OrderLine ol = new OrderLine();

            this.orderLines.Add(ol);
            ol.Order = this;

            ol.Product = p;
            ol.Quantity = quantity;

            if (p.ListPrice != 0)
            {
                float delta = p.ListPrice * quantity;

                this.amount += delta;

                this.OnAmountChanged(delta);
            }
        }

        public virtual void Cancel()
        {
        }
    }

    public class OrderAmountChangedEventArgs : EventArgs
    {
        private float delta;
        public float Delta
        {
            get { return delta; }
        }

        public OrderAmountChangedEventArgs(float delta)
        {
            this.delta = delta;
        }
    }
}
