using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Domain;
using NHibernate;

namespace OrderPersistenceTests
{
    [TestClass]
    public class DBTests
    {
        IProduct first = null;
        IProduct second = null;

        [TestInitialize]
        public void Init()
        {
            NHibernate.Cfg.Configuration config = new NHibernate.Cfg.Configuration();
            config.Configure();

            sf = config.BuildSessionFactory();

            CreateProducts();
        }
        private NHibernate.ISessionFactory sf;

        private void CreateProducts()
        {
            using (ISession s = sf.OpenSession())
            using (ITransaction tx = s.BeginTransaction())
            {
                first = s.Create<IProduct>();
                first.Name = "first";
                first.ListPrice = 100;

                second = s.Create<IProduct>();
                second.Name = "second";
                second.ListPrice = 200;

                s.Save(first);
                s.Save(second);

                tx.Commit();
            }
        }

        [TestMethod]
        public void CRUD()
        {
            ICustomer c = this.CreateCustomer("udi");

            c = this.UpdateCustomer(c.Id);

            using (ISession s = sf.OpenSession())
            {
                ICustomer cust = s.Get<ICustomer>(c.Id);
                foreach (IOrder o in cust.Orders)
                    foreach (IOrderLine ol in o.OrderLines)
                        Assert.IsTrue(ol.Quantity != 0);
            }
        }

        public ICustomer CreateCustomer(string name)
        {
            using (ISession s = sf.OpenSession())
            using (ITransaction tx = s.BeginTransaction())
            {
                ICustomer c = s.Create<ICustomer>();

                c.Name = name;
                IOrder o = c.CreateOrder();

                o.AddProduct(first, 11F);

                s.Save(c);

                tx.Commit();

                return c;
            }
        }

        public ICustomer UpdateCustomer(Guid id)
        {
            using (ISession s = sf.OpenSession())
            using (ITransaction tx = s.BeginTransaction())
            {
                ICustomer cust = s.Get<ICustomer>(id);

                IOrder o = cust.CreateOrder();
                o.AddProduct(second, 5F);

                tx.Commit();

                return cust;
            }
        }
    }
}
